/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.web;

import ch.hortis.sonar.core.Logs;
import org.picocontainer.Characteristics;
import org.picocontainer.MutablePicoContainer;
import org.sonar.commons.DaoFacade;
import org.sonar.commons.database.DatabaseConnector;
import org.sonar.core.MetricDatabaseSynchronizer;
import org.sonar.core.batch.Batch;
import org.sonar.core.batch.DatabaseListener;
import org.sonar.core.rules.RulesFacade;
import org.sonar.core.rules.RulesRepositorySynchronizer;
import org.sonar.plugins.api.Language;
import org.sonar.plugins.api.Plugin;
import org.sonar.plugins.api.PluginsIntrospector;
import org.sonar.plugins.api.RulesRepository;

import java.util.Map;

public class OperationalDatabaseListener implements DatabaseListener {
  private DatabaseConnector databaseConnector;
  private MutablePicoContainer picoContainer;
  private boolean started = false;

  public OperationalDatabaseListener(MutablePicoContainer picoContainer, DatabaseConnector databaseConnector) {
    this.databaseConnector = databaseConnector;
    this.picoContainer = picoContainer;
  }

  public boolean canStart() {
    return databaseConnector.isOperational();
  }

  public void start() {
    if (canStart()) {
      doStart();
      started = true;
    }
  }

  public boolean isStarted() {
    return started;
  }

  protected void doStart() {
    startDatabaseServices();
    registerMetrics();
    registerPlugins();
    startBatch();
  }

  private void startDatabaseServices() {
    picoContainer.as(Characteristics.NO_CACHE).addComponent(DaoFacade.class); // a new instance at each call
    picoContainer.as(Characteristics.NO_CACHE).addComponent(JRubyFacade.class); // a new instance at each call
    picoContainer.as(Characteristics.NO_CACHE).addComponent(RulesFacade.class); // a new instance at each call
  }

  private void registerMetrics() {
    Logs.info("loading metrics...");
    DaoFacade dao = picoContainer.getComponent(DaoFacade.class);
    try {
      MetricDatabaseSynchronizer synchronizer = new MetricDatabaseSynchronizer(dao);
      synchronizer.registerCoreMetrics();
      synchronizer.registerPlugins(picoContainer.getComponent(PluginsIntrospector.class).getPlugins());


    } finally {
      dao.stop();
    }
  }

  private void registerPlugins() {
    DaoFacade dao = picoContainer.getComponent(DaoFacade.class);
    try {
      RulesRepositorySynchronizer synchronizer = new RulesRepositorySynchronizer(dao);
      PluginsIntrospector introspector = picoContainer.getComponent(PluginsIntrospector.class);

      registerRules(synchronizer, introspector);
      registerProfiles(synchronizer, introspector);

    } finally {
      dao.stop();
    }
  }

  protected void registerRules(RulesRepositorySynchronizer synchronizer, PluginsIntrospector introspector) {
    for (Map.Entry<String, RulesRepository> entry : introspector.getRulesRepositoriesByPluginKey().entrySet()) {
      Logs.info("loading {} rules...", entry.getKey());
      synchronizer.registerRules(entry.getKey(), entry.getValue().getInitialReferential());
      synchronizer.registerRuleExtensions(entry.getKey(), entry.getValue());
    }
  }

  protected void registerProfiles(RulesRepositorySynchronizer synchronizer, PluginsIntrospector introspector) {
    synchronizer.cleanUpProvidedProfiles();
    for (Plugin plugin : introspector.getPlugins()) {
      RulesRepository rulesRepository = introspector.getRulesRepository(plugin.getKey());
      if (rulesRepository != null && rulesRepository.getProvidedProfiles() != null) {
        Logs.info("loading {} profiles...", plugin.getKey());
        synchronizer.registerProvidedProfiles(plugin, rulesRepository.getProvidedProfiles());
      }
    }
    for (Language language : introspector.getLanguages()) {
      Logs.info("activating {} default profile...", language.getKey());
      synchronizer.activateDefaultProfile(language);
    }
  }

  private void startBatch() {
    picoContainer.as(Characteristics.CACHE).addComponent(Batch.class);
    picoContainer.getComponent(Batch.class).start(); //not yet started because the container is already started
  }
}
